# -*- coding: UTF-8 -*-
# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@legco.biz> 
__version__ = '$Revision: 1.37.2.16 $'[11:-2]

import traceback, copy
from Acquisition import aq_parent, aq_base, aq_inner
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo, getSecurityManager
from Products.Archetypes.public import *
from Products.Archetypes.interfaces.referenceable import IReferenceable
from Products.CMFCore.CMFCorePermissions import \
     View, ModifyPortalContent, ManagePortal, RequestReview
from Products.CMFCore.utils import format_stx, getToolByName
from DocumentTemplate.DT_Util import html_quote
from AccessControl.PermissionRole import rolesForPermissionOn
from OFS.Image import File

from config import plone_ver
from utils import forum_parser
from ForumPermissions import *
from ForumFileField import ForumFileField

try:
    from Products.Archetypes.Field import decode, encode
except ImportError, e:
    def encode(value, instance, **kwargs):
        return value
    def decode(value, instance, **kwargs):
        return value


schema = BaseSchema + Schema((
    StringField('title', required=1,
                write_permission=ModifyReply,
                widget=StringWidget(label_msgid='label_title',
                                    description_msgid='help_title',
                                    i18n_domain='plone')),
    StringField('content', default='',
                write_permission=ModifyReply,
                widget=VisualWidget(label='Body text')),
    ImageField('image',
               validators=('checkImageSize',),
               write_permission=ModifyReply,
               allowable_content_types=('image/*',),
               sizes={'thumbnail': (128, 128),},
               widget=ImageWidget(label='Image')),
    ForumFileField('file',
                   validators=('checkFileSize',),
                   write_permission=ModifyReply,
                   widget=FileWidget(label='File')),
    StringField('message_icon',
                write_permission=ModifyReply,
                widget=StringWidget(label='Message icon')),
    StringField('text_format', default='text/plain',
                required=0,
                write_permission=ModifyReply,
                widget=StringWidget()),
    BooleanField('parsing_flag', default=1,
                 write_permission=ModifyReply,
                 widget=SelectionWidget()),
    StringField('text', default='',
                accessor='EditableBody',
                mutator='setText',
                write_permission=ModifyReply,
                widget=VisualWidget(label='Body text')),
    IntegerField('signature', default=1,
                 write_permission=ModifyReply,
                 widget=SelectionWidget()),
    ))

class ForumMessage(BaseContent):
    """fcForum Object Class Python"""
    meta_type = 'ForumMessage'
    archetype_name = 'Forum Message'

    schema = schema

    global_allow = 0
    content_icon = 'discussionitem_icon.gif'

    if plone_ver == 1:
        actions = (
            { 'id': 'view',
              'name': 'View',
              'action': 'forum_message',
              'permissions': (View,)
              },
            { 'id': 'edit',
              'name': 'Edit',
              'action': 'portal_form/forum_message_edit',
              'permissions': (ModifyReply,)
              },
            { 'id': 'reply',
              'name': 'Reply',
              'action': 'portal_form/forum_reply_form',
              'permissions': (AddTopicReply,)
              },
            { 'id': 'preview',
              'name': 'Preview',
              'action': 'forum_message_preview',
              'permissions': (View,)
              },
            { 'id': 'delete',
              'name': 'Delete',
              'action': 'portal_form/forum_message_delete_form',
              'permissions': (ModifyReply,)
              },
            )
    else:
        actions = (
            { 'id': 'view',
              'name': 'View',
              'action': 'string:$object_url/forum_message',
              'permissions': (View,)
              },
            { 'id': 'edit',
              'name': 'Edit',
              'action': 'string:$object_url/portal_form/forum_message_edit',
              'permissions': (ModifyReply,)
              },
            { 'id': 'reply',
              'name': 'Reply',
              'action': 'string:$object_url/portal_form/forum_reply_form',
              'permissions': (AddTopicReply,)
             },
            { 'id': 'preview',
              'name': 'Preview',
              'action': 'string:$object_url/forum_message_preview',
              'permissions': (View,)
              },
            { 'id': 'delete',
              'name': 'Delete',
              'action': 'string:$object_url/portal_form/forum_message_delete_form',
              'permissions': (ModifyReply,)
              },
            )

    security = ClassSecurityInfo()
    security.setPermissionDefault(RequestReview, ['replyOwner'])
    security.declareProtected(ModifyReply, 'processForm')

    content = ''
    text_format = 'text/plain'
    signature = 1
    parsing_flag = 1
    message_icon = ''

    security.declareProtected(RequestReview, '_test_test')
    def _test_test(self):
        pass

    def _setId(self, *args, **kwargs):
        pass

    def _canCopy(self, op=0):
        return 0

    def initializeArchetype(self, **kwargs):
        if kwargs.has_key('file'):
            try:
                kwargs['file'].seek(0)
            except:
                pass

        try:
            if kwargs.has_key('parent'):
                parent = kwargs['parent']
                if IReferenceable.isImplementedBy(parent):
                    self.addReference(parent, 'parent')
        except:
            pass

        BaseContent.initializeArchetype(self, **kwargs)

    _saved_refs = 0
    _set_owner = 1
    def manage_afterAdd(self, item, container):
        BaseContent.manage_afterAdd(self, item, container)

        # restore references
        try:
            refs = self._refs
        except:
            refs = []

        atool = self.archetype_tool
        for r in refs:
            atool.addReference(self, r[0], r[1])
        self._refs = ()
        self._saved_refs = 0

        # update member posts count
        if not (hasattr(item, '_copy') and item._copy):
    	    self.manage_permission(
                ModifyPortalContent, ('Anonymous', 'Owner', replyOwner), 0)

            mtool = self.portal_membership
            user = mtool.getAuthenticatedMember()

            if not mtool.isAnonymousUser() and user:
                posts = user.getProperty('fc_posts', 0) + 1
                user.setMemberProperties({'fc_posts': posts})

                if self._set_owner:
                    self.manage_permission(RequestReview, (replyOwner,), 0)
                    self.manage_setLocalRoles(user.getId(), [replyOwner,])

            self.creator = str(user)


    def manage_beforeDelete(self, item, container):
        if not self._saved_refs:
            self._refs = copy.copy(self.archetype_tool.refs.get(self.UID(), []))
            self._saved_refs = 1

        # check if remove or move
        if not (hasattr(item, '_copy') and item._copy):
            try:
                # remove children
                brefs = self.getBRefs('parent')
                if type(brefs) is not (type([]) or type(())):
                    brefs = [brefs,]
                children = [ob.getId() for ob in brefs]
                aq_parent(self).manage_deleteReply(children)
            except:
                pass
            
            mtool = self.portal_membership
            user = mtool.getMemberById(self.Creator())
            
            if user is not None:
                posts = user.getProperty('fc_posts', 1) - 1
                if posts >= 0:
                    user.setMemberProperties({'fc_posts': posts})

        BaseContent.manage_beforeDelete(self, item, container)

    security.declareProtected(DeleteReply, 'manage_deleteMessage')
    def manage_deleteMessage(self, REQUEST=None):
        """ """
        try:
            aq_parent(self).manage_deleteReply([self.getId()])
            msg = '/?portal_status_message=Reply+deleted'
            if REQUEST is not None:
                return REQUEST.RESPONSE.redirect('%s%s'%(
                    aq_parent(self).absolute_url(), msg))
        except Exception, e:
            msg = '/?portal_status_message=%s'%e
            if REQUEST is not None:
                return REQUEST.RESPONSE.redirect('%s%s'%(self.absolute_url(), msg))

    security.declareProtected(View, 'Title')
    def Title(self):
        """ """
        return self.getField('title').get(self)

    security.declareProtected(View, 'Description')
    def Description(self):
        """ """
        desc = self.getField('description').get(self)
        if not desc:
            desc = self.description
            if not desc:
                desc = self.Content(add_breaks=0, preview=1)[:150]
        return desc

    security.declareProtected(View, 'Content')
    def Content(self, emoticons=1, add_breaks=1, preview=None, strip_html=0):
        """ """
        return forum_parser(
            self, self.getField('content').get(self),
            emoticons, 0, preview,
            strip_html = strip_html, text_format=self.text_format)

    security.declareProtected(View, 'CookedBody')
    def CookedBody(self):
        """ """
        try:
            if hasattr(aq_base(self), 'cooked_text'):
                return encode(self.cooked_text, self)
            else:
                self.setText(self.content)
                return encode(self.cooked_text, self)
        except:
            return self.getField('content').get(self)

    security.declareProtected(View, 'EditableBody')
    def EditableBody(self):
        """ """
        return self.getField('content').get(self)

    security.declareProtected(ModifyReply, 'setText')
    def setText(self, value, mimetype='', **kwargs):
        """ """
        text_format = self.REQUEST.get('text_format', self.text_format)
        self.text_format = text_format

        parsing_flag = self.REQUEST.get('parsing_flag', self.parsing_flag)

        self.getField('content').set(self, value, **kwargs)
        self.cooked_text = decode(forum_parser(
            self, value, text_format=text_format, parsing=parsing_flag),
                                  self, **kwargs)

        self.getField('description').set(
            self, self.Content(add_breaks=0, preview=1)[:150])

    security.declareProtected(View, 'Creator')
    def Creator(self):
        """ """
        try:
            return self.creator
        except:
            return BaseContent.Creator(self)
        
    security.declareProtected(View, 'getParent')
    def getParent(self):
        """ """
        obs = self.getRefs('parent')
        try:
            return obs[0]
        except:
            return None

    security.declareProtected(View, 'getChildren')
    def getChildren(self, sort_on='created', reverse=0):
        """ """
        seq = []
        brefs = self.getBRefs('parent')
        if type(brefs) is not (type([]) or type(())):
            brefs = [brefs,]
        for ob in brefs:
            if ob is not None:
                attr = getattr(ob, sort_on)
                seq.append((attr(), ob))

        seq.sort()
        if reverse:
            seq.reverse()
        return [r[1] for r in seq]

    security.declareProtected(View, 'hasImage')
    def hasImage(self):
        """ """
        if hasattr(aq_base(self), 'image') and (self.image.getSize() > 0):
            return 1
        else:
            return 0

    security.declareProtected(View, 'getImage')
    def getImage(self):
        """ """
        if hasattr(aq_base(self), 'image'):
            return self.image
        else:
            return None

    security.declareProtected(View, 'SearchableText')
    def SearchableText(self):
        """ """
        return '%s %s'%(self.Title(), self.EditableBody())

    security.declareProtected(View, 'getMessage')
    def getMessage(self):
        """ """
        return aq_inner(self)

    security.declareProtected(View, 'getSignature')
    def getSignature(self):
        """ """
        signature = self.signature
        if signature:
            request = self.REQUEST
            signature = signature - 1
            mtool = getToolByName(self, 'portal_membership')

            member = mtool.getMemberById(self.Creator())
            if not member:
                return ''
            member_name = member.getUserName()

            try:
                cache = request._sigs_cache
            except:
                cache = {}
                request._sigs_cache = cache

            if cache.has_key(member_name):
                member_cache = cache[member_name]
            else:
                member_cache = {}
                cache[member_name] = member_cache

            try:
                return member_cache[signature]
            except:
                pass

            sigs = member.getProperty('signatures', [])
            try:
                sig = sigs[signature]
            except:
                sig = ''

            member_cache[signature] = sig

            return sig
        else:
            return ''

    security.declareProtected(View, 'notifyStateChanged')
    def notifyStateChanged(self, sci):
        """ """
        pass


registerType(ForumMessage)

